unit uService;

{******************************************************************************
Initial Author   :    Scott Price
Initially Created:    2003-09-10
Language         :    Object Pascal
Compiler Version :    Delphi 3.02, 4.03, 5.01, 6.02 (UK)
Copyright        :     Information Technology Sector Limited, 1996-2002

Source Code Constraints:
========================
   Distribution, disclosure or modifications of this source code, outside of
   any licenced agreements with Information Technology Sector Limited, or
   removal of this notice constitutes a breach of any or all license agreements.

Operating Systems Tested On:
============================
   Microsoft Windows(R) 98, First Edition
   Microsoft Windows(R) 2000, Service Pack 2

Design Notes:
=============

VERSION  AUTHOR                DATE/TIME
===============================================================================
$history$
$nokeywords$
===============================================================================
NOTES:
===============================================================================
TODO:
******************************************************************************}


interface



uses
  { Own Units }
  uJclExceptInfo,  { This is the unit containing the encapsulation of the JclDebug use. }

  { Borland Standard Units }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs;


const
  UM_NEW_EXCEPTION_INFO = WM_USER + 1369;


type
  { TsvcErrorMaker }

  TsvcErrorMaker = class(TService)
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure ServiceExecute(Sender: TService);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
  private
    { Private declarations }
    FList: TList;
    FException: Boolean;
    { This Object is based on the Class that Encapsulates the use of the JclDebug facilities }
  {$IFDEF STANDARD_EXCEPT}
    FExceptionStuff: uJclExceptInfo.TStandardExceptionInfo;
  {$ELSE}
    FExceptionStuff: uJclExceptInfo.TAdvancedExceptionInfo;
  {$ENDIF}
    FExceptionDataFile: String;
    procedure DisplayExceptionInfo(Sender: TObject; const Info: TStrings);
    procedure SaveInfoToExceptionData(const Info: TStrings);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetServiceController: TServiceController; override;
  end;


var
  svcErrorMaker: TsvcErrorMaker;



implementation



{$R *.DFM}


uses
  { Own Units }
  uAPIUtils;           


const
  icSleepTime = 6000;  { 6 secs }


procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  svcErrorMaker.Controller(CtrlCode);
end;


  { TsvcErrorMaker }

constructor TsvcErrorMaker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FList := Nil;
  FException := False;

  { Here, as you can see from the commented out line, I have created it in such
    a way that you can provide an exception handler in the CreateHooked
    constructor that automatically sets up the handler.  Your choice as to your
    preferred use }
  { FExceptionStuff := TStandardExceptionInfo.CreateHooked(Self.DisplayExceptionInfo); }
  { FExceptionStuff := TStandardExceptionInfo.Create; - Old Line }
  FExceptionDataFile := ChangeFileExt(uAPIUtils.ModuleFileName(HInstance), '.log');

{$IFDEF STANDARD_EXCEPT}
  FExceptionStuff := TStandardExceptionInfo.Create;
{$ELSE}
  FExceptionStuff := TAdvancedExceptionInfo.Create;
  FExceptionStuff.ShowAllFrames := True;
{$ENDIF}

  FExceptionStuff.OnException := Self.DisplayExceptionInfo;

  FExceptionStuff.HookExceptions;    { << Important - This starts tracing
                                                      those stack exceptions.
                                                      Remove this and you will
                                                      get nothing at all out }
end;

destructor TsvcErrorMaker.Destroy;
begin
  TRY
    if Assigned(FExceptionStuff) then
    begin
      FExceptionStuff.UnHookExceptions;  { << Stops this class using the
                                              JclDebug monitoring of Exceptions }
      FreeAndNil(FExceptionStuff);
    end;

    if Assigned(FList) then
      FreeAndNil(FList);
  FINALLY
    inherited Destroy;
  END;
end;

function TsvcErrorMaker.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TsvcErrorMaker.DisplayExceptionInfo(Sender: TObject; const Info: TStrings);
var
  hViewer: THandle;
begin
  { TODO:  Write out to a file and notify main form to reload it }
  FException := True;

{$IFNDEF STANDARD_EXCEPT}
  if NOT FExceptionStuff.Handled then
    SaveInfoToExceptionData(Info);
{$ELSE}
  SaveInfoToExceptionData(Info);
{$ENDIF}                        

  hViewer := FindWindow('fmJclDebugNonVisualFileViewer', Nil);
  if hViewer <> 0 then
    PostMessage(hViewer, UM_NEW_EXCEPTION_INFO, 0, 0);
end;

procedure TsvcErrorMaker.SaveInfoToExceptionData(const Info: TStrings);
var
  fs: TFileStream;
begin
  if FileExists(FExceptionDataFile) then
    fs := TFileStream.Create(FExceptionDataFile, fmOpenWrite)
  else
    fs := TFileStream.Create(FExceptionDataFile, fmCreate);

  if Assigned(fs) then
    TRY
      fs.Position := fs.Size;

      Info.SaveToStream(fs);
    FINALLY
      FlushFileBuffers(fs.Handle);
      FreeAndNil(fs);
    END;
end;

procedure TsvcErrorMaker.ServiceStart(Sender: TService;
  var Started: Boolean);
begin
  Started := True;
end;

procedure TsvcErrorMaker.ServiceExecute(Sender: TService);
begin
  { Pause initially for a short period }
  Sleep(icSleepTime);

  while not Terminated do
  begin
    if NOT FException then
    begin
      try
        FList.Add(Self);
      except
      end;
    end else
      { When we have had one exception and written out the data, finish the
        threads execution and stop the service }
      Break;
  end;
end;

procedure TsvcErrorMaker.ServiceStop(Sender: TService;
  var Stopped: Boolean);
begin
  Stopped := True;
end;

end.
